// Author(s) : Justin Crooks
// Copyright 2012, Test Equipment Plus
//
// Description :
// 
// Date : Author of Revision : Description (if any)
// 15 MAY 2012 : AJ Montgomery : Added Headers

#include "stdafx.h"
#include "SHAPISampleApp.h"

#include "SHAPISampleAppDoc.h"
#include "SHAPI.h"
#include "SlowSweepDlg.h"
#include "FastSweepDlg.h"
#include "ConfigDlg.h"
#include "ZSFreqDlg.h"
#include <cmath>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define PI 3.14159265358979
#define INV2PI 0.1591549431
#define MAX_IQ_SAMPLE_RATE 486111.1111
// CSHAPISampleAppDoc

IMPLEMENT_DYNCREATE(CSHAPISampleAppDoc, CDocument)

BEGIN_MESSAGE_MAP(CSHAPISampleAppDoc, CDocument)
	ON_COMMAND(ID_COMMAND_INITIALIZE, &CSHAPISampleAppDoc::OnCommandInitialize)
	ON_UPDATE_COMMAND_UI(ID_COMMAND_INITIALIZE, &CSHAPISampleAppDoc::OnUpdateCommandInitialize)
	ON_COMMAND(ID_COMMAND_CONFIGURE, &CSHAPISampleAppDoc::OnCommandConfigure)
	ON_UPDATE_COMMAND_UI(ID_COMMAND_CONFIGURE, &CSHAPISampleAppDoc::OnUpdateCommandConfigure)
	ON_COMMAND(ID_COMMAND_SLOWSWEEP, &CSHAPISampleAppDoc::OnCommandSlowsweep)
	ON_UPDATE_COMMAND_UI(ID_COMMAND_SLOWSWEEP, &CSHAPISampleAppDoc::OnUpdateCommandSlowsweep)
	ON_COMMAND(ID_COMMAND_FASTSWEEP, &CSHAPISampleAppDoc::OnCommandFastsweep)
	ON_UPDATE_COMMAND_UI(ID_COMMAND_FASTSWEEP, &CSHAPISampleAppDoc::OnUpdateCommandFastsweep)
	ON_COMMAND(ID_COMMAND_CLOSE, &CSHAPISampleAppDoc::OnCommandClose)
	ON_UPDATE_COMMAND_UI(ID_COMMAND_CLOSE, &CSHAPISampleAppDoc::OnUpdateCommandClose)
	ON_COMMAND(ID_I_SHOWRAW, &CSHAPISampleAppDoc::OnIShowraw)
	ON_COMMAND(ID_I_SHOWFMDEMOD, &CSHAPISampleAppDoc::OnIShowfmdemod)
	ON_COMMAND(ID_I_SHOWAMDEMOD, &CSHAPISampleAppDoc::OnIShowamdemod)
	ON_UPDATE_COMMAND_UI(ID_I_SHOWAMDEMOD, &CSHAPISampleAppDoc::OnUpdateIShowamdemod)
	ON_UPDATE_COMMAND_UI(ID_I_SHOWFMDEMOD, &CSHAPISampleAppDoc::OnUpdateIShowfmdemod)
	ON_UPDATE_COMMAND_UI(ID_I_SHOWRAW, &CSHAPISampleAppDoc::OnUpdateIShowraw)
	ON_COMMAND(ID_COMMAND_TRACKINGGENERATORSWEEP, &CSHAPISampleAppDoc::OnCommandTrackinggeneratorsweep)
	ON_UPDATE_COMMAND_UI(ID_COMMAND_TRACKINGGENERATORSWEEP, &CSHAPISampleAppDoc::OnUpdateCommandTrackinggeneratorsweep)
END_MESSAGE_MAP()


// CSHAPISampleAppDoc construction/destruction

CSHAPISampleAppDoc::CSHAPISampleAppDoc()
{
	m_bIsInitialized = false;
	m_bIsConfigured = false;
	m_LastCommand = "Please select 'Initialize' from the command menu";
	m_LastResult = "(No function selected)";

	m_HavePlot = false;
	m_PlotData = NULL;
	m_HasTG = false;
	m_plotType = PLOT_TYPE_DBM;

}

CSHAPISampleAppDoc::~CSHAPISampleAppDoc()
{
}

BOOL CSHAPISampleAppDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}




// CSHAPISampleAppDoc serialization

void CSHAPISampleAppDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}


// CSHAPISampleAppDoc diagnostics

#ifdef _DEBUG
void CSHAPISampleAppDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CSHAPISampleAppDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


// CSHAPISampleAppDoc commands

void CSHAPISampleAppDoc::OnCommandInitialize()
{

	m_LastCommand = "Initializing";

	int retval1 = SHAPI_Initialize();  // - return 0

	m_LastResult.Format(_T("SHAPI_Initialize returned %d"),retval1);
	UpdateAllViews(NULL);

	m_bIsInitialized = true;
	if(SHAPI_SetTGFreqAtten(0.0,30)==0)//Turn it off, if it is present.  Otherwise an error code is returned 
		m_HasTG = true;
}

void CSHAPISampleAppDoc::OnUpdateCommandInitialize(CCmdUI *pCmdUI)
{
	//Allow initialization only once
	if(m_bIsInitialized) pCmdUI->Enable(0);
	else pCmdUI->Enable();

	
}

void CSHAPISampleAppDoc::OnCommandConfigure()
{
	CConfigDlg dlg;
	if(dlg.DoModal() != IDOK) return; //Exit on cancel

	int decimation = (int)(pow(2.0,dlg.m_DecimationIdx));

	int retval = SHAPI_Configure(5.0 * dlg.m_AttenIdx, dlg.m_MixerIdx, dlg.m_SensitivityIdx, decimation);
	m_decimation = decimation;
	m_LastResult.Format(_T("SHAPI_Configure returned %d"),retval);
	UpdateAllViews(NULL);

	m_bIsConfigured=true;
}

void CSHAPISampleAppDoc::OnUpdateCommandConfigure(CCmdUI *pCmdUI)
{
	//Allow configuration only after initialization
	if(m_bIsInitialized) pCmdUI->Enable();
	else pCmdUI->Enable(0);
}

void CSHAPISampleAppDoc::OnCommandSlowsweep()
{
	CSlowSweepDlg dlg;
	if(dlg.DoModal() != IDOK) return; //Exit on cancel

	double startFreq = dlg.m_CenterFreq * 1.0e6 - dlg.m_SpanFreq * 1.0e3 * 0.5; //Center - 1/2 span.  Span in KHz
	double stopFreq = dlg.m_CenterFreq * 1.0e6 + dlg.m_SpanFreq * 1.0e3 * 0.5; //Center + 1/2 span
	int fftsz = (int)(pow(2.0,dlg.m_FFTIndex + 6));//Index 0, FFT size 64.

	int sweepsize = SHAPI_GetSlowSweepCount(startFreq,stopFreq,fftsz); //How big will the trace be?
	FreeAllocPlotMemory(sweepsize);

	int returnCount = sweepsize;
	//Average data when it is convenient to do so.
	int avgCount = 16;
	if(fftsz > 1024) avgCount = 16384 / fftsz;
	if(avgCount<1) avgCount=1;

	int retval = SHAPI_GetSlowSweep(m_PlotData,startFreq,stopFreq,returnCount,fftsz,avgCount);
	m_PlotSize = returnCount;
	m_HavePlot = true; //We have a plot.
	m_plotType=PLOT_TYPE_DBM;

	m_LastResult.Format(_T("SHAPI_GetSlowSweep returned %d. SweepSize = %d"),retval,sweepsize);
	UpdateAllViews(NULL);

}

void CSHAPISampleAppDoc::OnUpdateCommandSlowsweep(CCmdUI *pCmdUI)
{
	//Allow sweep only after configuration
	if(m_bIsConfigured) pCmdUI->Enable();
	else pCmdUI->Enable(0);
}

void CSHAPISampleAppDoc::OnCommandFastsweep()
{
	CFastSweepDlg dlg;
	if(dlg.DoModal() != IDOK) return; //Exit on cancel

	double startFreq = dlg.m_CtrFreq * 1.0e6 - dlg.m_SpanFreq * 1.0e6 * 0.5; //Center - 1/2 span.  Span in MHz here
	double stopFreq = dlg.m_CtrFreq * 1.0e6 + dlg.m_SpanFreq * 1.0e6 * 0.5; //Center + 1/2 span
	int fftsz = (int)(pow(2.0,dlg.m_FFTIdx + 4));//Index 0, FFT size 16.

	int sweepsize = SHAPI_GetFastSweepCount(startFreq,stopFreq,fftsz);
	FreeAllocPlotMemory(sweepsize);

	int returnCount = sweepsize;

	//Average data when it is convenient to do so.

	int retval = SHAPI_GetFastSweep(m_PlotData,startFreq,stopFreq,returnCount,fftsz);
	m_PlotSize = returnCount;
	m_HavePlot = true; //We have a plot.
	m_plotType=PLOT_TYPE_DBM;

	m_LastResult.Format(_T("SHAPI_GetSlowSweep returned %d. SweepSize = %d"),retval,sweepsize);
	UpdateAllViews(NULL);

}

void CSHAPISampleAppDoc::OnUpdateCommandFastsweep(CCmdUI *pCmdUI)
{
	//Allow sweep only after configuration
	if(m_bIsConfigured) pCmdUI->Enable();
	else pCmdUI->Enable(0);
}

void CSHAPISampleAppDoc::FreeAllocPlotMemory(int newsize)
{
	m_HavePlot = false; //WE no longer have a plot if we are deleting it.

	if(m_PlotData != NULL)
		delete m_PlotData;

	m_PlotData = new double[newsize];
	m_PlotSize = newsize;

}

void CSHAPISampleAppDoc::OnCommandClose()
{
	SHAPI_Reset();
	m_bIsInitialized = false;
}

void CSHAPISampleAppDoc::OnUpdateCommandClose(CCmdUI *pCmdUI)
{
	if(m_bIsInitialized) pCmdUI->Enable();
	else pCmdUI->Enable(0);
}

void CSHAPISampleAppDoc::OnIShowraw()  //Raw I / Q Data
{
	int i;
	if(m_plotType==PLOT_TYPE_DBM) m_plotType=PLOT_TYPE_INT; //Default will be to display I and Q values of last 256 bytes as int

	CZSFreqDlg dlg;
	if(dlg.DoModal() != IDOK) return; //Exit on cancel

	const int datasize = 4096;
	const int plotsize = 512;	//only show a small part of data collected
	int * pIData = new int[datasize];
	int * pQData = new int[datasize];

	double myFrequency = dlg.m_freq * 1.0e6;
	SHAPI_GetIQDataPacket(pIData,pQData,myFrequency,datasize);
	CString s;
	s.Format("Actual Center Frequency: %f Hz",myFrequency);
	MessageBox(NULL,s,"I/Q Center Freq",MB_OK);

	FreeAllocPlotMemory(plotsize);
	if(m_plotType==PLOT_TYPE_INT) //Raw I/Q data, I in first half, Q in second half
	{
		for(i=0; i<256; i++)
			m_PlotData[i] = pIData[i+datasize-256];
		for(i=0; i<256; i++)
			m_PlotData[i+256] = pQData[i+datasize-256];
	}

	if(m_plotType==PLOT_TYPE_HZ) //FM demodulate, start at sample 3000,
	{
		double thisPhase; 
		double lastPhase=atan2( (double)pQData[2999], (double)pIData[2999]);
		double instFreq;
		for(i=0; i<plotsize; i++)
		{
			thisPhase = atan2((double)pQData[i+3000], (double)pIData[i+3000]);
			instFreq = lastPhase - thisPhase;
			if(instFreq > PI) instFreq -= 2*PI;
			if(instFreq < (-PI)) instFreq += 2*PI;
			lastPhase = thisPhase;
			m_PlotData[i] = instFreq * INV2PI * MAX_IQ_SAMPLE_RATE / m_decimation; //Convert to Hz offset
		}
	}

	if(m_plotType==PLOT_TYPE_PCT) //Percent AM modulate, start at sample 3000,
	{
		double AMsum = 0.0;
		for(i=0; i<plotsize; i++)
		{
			m_PlotData[i] = sqrt( (double)pQData[i+3000] * pQData[i+3000] + pIData[i+3000] * pIData[i+3000] ); //DEMOD is now AM
			AMsum += m_PlotData[i];
		}

		double topct = plotsize * 100.0 / AMsum;

			for(i=0; i<plotsize; i++)
				m_PlotData[i] *= topct; // Normalize to 100%
	}

	m_HavePlot = true; //We have a plot.
	UpdateAllViews(NULL);

	delete pIData;
	delete pQData;
}

void CSHAPISampleAppDoc::OnIShowfmdemod()
{
	m_plotType=PLOT_TYPE_HZ;
	OnIShowraw();
}

void CSHAPISampleAppDoc::OnIShowamdemod()
{
	m_plotType=PLOT_TYPE_PCT;
	OnIShowraw();
}

void CSHAPISampleAppDoc::OnUpdateIShowamdemod(CCmdUI *pCmdUI)
{
	if(m_bIsConfigured) pCmdUI->Enable();
	else pCmdUI->Enable(0);
}

void CSHAPISampleAppDoc::OnUpdateIShowfmdemod(CCmdUI *pCmdUI)
{
	if(m_bIsConfigured) pCmdUI->Enable();
	else pCmdUI->Enable(0);
}

void CSHAPISampleAppDoc::OnUpdateIShowraw(CCmdUI *pCmdUI)
{
	if(m_bIsConfigured) pCmdUI->Enable();
	else pCmdUI->Enable(0);
}

void CSHAPISampleAppDoc::OnCommandTrackinggeneratorsweep()
{
	const int sweepsize = 1000;
	double dbThruData[sweepsize];
	double dbDUTData[sweepsize];

	MessageBox(NULL,"We will sweep from 1 to 1000 MHz,\r\n in 1 MHz steps","NOTE",MB_OK);
	MessageBox(NULL,"Connect THRU (connect the TG44 output to the\r\n SA44 input and make sure a BNC cable ties\r\n the SYNCs together)","USER ACTION",MB_OK);

	SHAPI_ProcTGSweep(dbThruData, 1.0e6, 1.0e6, sweepsize, 0); // Note: 0 dB attenuator should be changed to a higher value for sweeping amplifiers

	MessageBox(NULL,"Insert DUT (Put the device to test\r\n between the TG44 and the SA44)","USER ACTION",MB_OK);

	SHAPI_ProcTGSweep(dbDUTData, 1.0e6, 1.0e6, sweepsize, 0); //

	FreeAllocPlotMemory(sweepsize);
	for(int i=1; i<sweepsize; i++)
		m_PlotData[i-1] = dbDUTData[i] - dbThruData[i];
	m_PlotSize = sweepsize-1; //First point appears to be bogus
	m_HavePlot = true; //We have a plot.
	m_plotType=PLOT_TYPE_DBM;

	m_LastResult.Format(_T("SHAPI_ProcTGSweep complete"));
	UpdateAllViews(NULL);

}

void CSHAPISampleAppDoc::OnUpdateCommandTrackinggeneratorsweep(CCmdUI *pCmdUI)
{
	if((m_bIsConfigured) && m_HasTG)
		pCmdUI->Enable();
	else 
		pCmdUI->Enable(0);
}
