// Author(s) : Justin Crooks
// Copyright 2012, Test Equipment Plus
//
// Description :
// 
// Date : Author of Revision : Description (if any)
// 15 MAY 2012 : AJ Montgomery : Added Headers

#include "stdafx.h"
#include "SHAPISampleApp.h"

#include "SHAPISampleAppDoc.h"
#include "SHAPISampleAppView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSHAPISampleAppView

IMPLEMENT_DYNCREATE(CSHAPISampleAppView, CView)

BEGIN_MESSAGE_MAP(CSHAPISampleAppView, CView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, &CView::OnFilePrintPreview)
END_MESSAGE_MAP()

// CSHAPISampleAppView construction/destruction

CSHAPISampleAppView::CSHAPISampleAppView()
{
	// TODO: add construction code here

}

CSHAPISampleAppView::~CSHAPISampleAppView()
{
}

BOOL CSHAPISampleAppView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CSHAPISampleAppView drawing

void CSHAPISampleAppView::OnDraw(CDC* pDC)
{
	CSHAPISampleAppDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	int LeftMargin = 10;
	int RightMargin = 639;
	int TopMargin = 80;
	int BottomMargin = 479;
	int i;

	//Erase last picture (640 x 480)
	pDC->FillSolidRect(0,0,RightMargin+1,BottomMargin+1,RGB(255,255,255)); //Paint white
	pDC->SetTextColor( RGB(0,0,0) );//BLK on WHT
	pDC->SetBkColor(RGB(255,255,255));

	pDC->TextOut(10,10,pDoc->m_LastCommand); //Space for 2 Command / Result strings
//	pDC->TextOut(10,40,pDoc->m_LastResult);

	//These margins are used by the plotting code, below
	CString line2Print = _T("Directions: Inject a 315 MHz signal at 0 dBm");  


	//Do not plot anything if we have no data.
	if(pDoc->m_HavePlot)
	{
		pDC->SelectStockObject(BLACK_PEN);

		pDC->MoveTo(LeftMargin-1,TopMargin-1);		//Graph perimeter
		pDC->LineTo(RightMargin+1,TopMargin-1);
		pDC->LineTo(RightMargin+1,BottomMargin+1);
		pDC->LineTo(LeftMargin-1,BottomMargin+1);
		pDC->LineTo(LeftMargin-1,TopMargin-1);

		double maxval = pDoc->m_PlotData[0];
		double minval = pDoc->m_PlotData[0];

		//Get max and min
		for(i=1; i<pDoc->m_PlotSize; i++) 
		{
			double currentVal = pDoc->m_PlotData[i];
			if(currentVal > maxval) maxval = currentVal;
			if(currentVal < minval) minval = currentVal;
		}

		if(pDoc->m_plotType==PLOT_TYPE_DBM)
			line2Print.Format("Trace Min: %.2f dBm, Max %.2f dBm", minval, maxval);
		else if(pDoc->m_plotType==PLOT_TYPE_HZ)
			line2Print.Format("Trace Min: %.2f Hz, Max %.2f Hz", minval, maxval);
		else if(pDoc->m_plotType==PLOT_TYPE_INT)
			line2Print.Format("Trace Min count: %.0f, Max count %.0f", minval, maxval);
		else if(pDoc->m_plotType==PLOT_TYPE_PCT)
			line2Print.Format("Trace Min PCT: %.1f, Max PCT %.1f", minval, maxval);

		pDC->TextOut(10,40,line2Print);

		//Move to origin
		pDC->MoveTo( LeftMargin, ScaleVal(pDoc->m_PlotData[0],minval,maxval,BottomMargin,TopMargin));
		
		//Scale & plot graph
		for(i=1; i<pDoc->m_PlotSize; i++) 
			pDC->LineTo( ScaleVal(i,0,pDoc->m_PlotSize,LeftMargin,RightMargin),
						 ScaleVal(pDoc->m_PlotData[i],minval,maxval,BottomMargin,TopMargin));
	}
	else // No plot, so let's display some instructions...
	{
		pDC->TextOut(10,40,pDoc->m_LastResult);

		pDC->TextOut(10,100,line2Print);
		line2Print = _T("Select Command-->Initialize");
		pDC->TextOut(20,130,line2Print);
		line2Print = _T("Wait 10 seconds");
		pDC->TextOut(30,160,line2Print);
		line2Print = _T("Select Command-->Configure");
		pDC->TextOut(40,190,line2Print);
		line2Print = _T("Select Command-->Slow Sweep");
		pDC->TextOut(50,220,line2Print);
		line2Print = _T("The plot will be shown here.");
		pDC->TextOut(60,250,line2Print);

		
	}

}

int CSHAPISampleAppView::ScaleVal(double valIn, double valMin, double valMax, int pixMin, int pixMax)
{
	return (int)( pixMin + (valIn-valMin) / (valMax-valMin) * (pixMax-pixMin) );
}


// CSHAPISampleAppView printing

BOOL CSHAPISampleAppView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CSHAPISampleAppView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CSHAPISampleAppView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// CSHAPISampleAppView diagnostics

#ifdef _DEBUG
void CSHAPISampleAppView::AssertValid() const
{
	CView::AssertValid();
}

void CSHAPISampleAppView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSHAPISampleAppDoc* CSHAPISampleAppView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSHAPISampleAppDoc)));
	return (CSHAPISampleAppDoc*)m_pDocument;
}
#endif //_DEBUG


// CSHAPISampleAppView message handlers
